/*
 * exBender.cpp:  Main driver for reading SMPS-files and solving.
 *
 *         Reads problem from SMPS files and solves using
 *         ekks_bendersLSolve().
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include "scn_c_api.h"

#define COLUMNS 0
#define ROWS    1
#define MXSCN 15000                                       /* maximum scenarios */


double exSMPS_Solve(const char *prob_Name);

int main( long argc, char *argv[] ) {

    double obj_Value;


/* Problem KandW3 */
    obj_Value = exSMPS_Solve( "KandW3");
    assert(fabs(obj_Value - 2613.0) < 1.0e-6*2613.0);

/* The following stanzas are ifdef'd out.
   To run them, change the 0 to 1, recompile,
   and run the exBender executable in the ../mpsdata directory */

#if 0

/* Problem CAP */
    obj_Value = exSMPS_Solve( "CAP");
    assert(fabs(obj_Value + 272.8345) < 1.0e-6*272.8345);

/* Problem LandS */
    obj_Value = exSMPS_Solve( "LandS");
    assert(fabs(obj_Value - 381.8533) < 1.0e-6*381.8533);

/* Problem PROD_MIX */
    obj_Value = exSMPS_Solve( "PROD_MIX");
    assert(fabs(obj_Value + 17730.32) < 1.0e-6*17730.32);

/* Problem gbd */
    obj_Value = exSMPS_Solve( "gbd");
    assert(fabs(obj_Value - 1566.042) < 1.0e-6*1566.042);

/* Problem hydro */
    obj_Value = exSMPS_Solve( "hydro");
    assert(fabs(obj_Value - 265.35) < 1.0e-6*265.35);

#endif

/* Print comforting message */
    printf("\n ************************************");
    printf("\n ***   All testing successful !   ***");
    printf("\n ************************************\n");fflush(NULL);
	return(0);
}

double exSMPS_Solve(const char *prob_Name)
{

	char core_Name[128];
	char time_Name[128];
	char stoch_Name[128];
	char spl_Name[128];
	strcpy(core_Name,prob_Name); strcat(core_Name,".core");
	strcpy(time_Name,prob_Name); strcat(time_Name,".time");
	strcpy(stoch_Name,prob_Name); strcat(stoch_Name,".stoch");
	strcpy(spl_Name,prob_Name); strcat(spl_Name,".spl");

/* Initialize EKKContext and EKKStoch */
    EKKContext *env=ekks_initializeContext();	
    EKKStoch *stoch=ekks_newStoch(env,NULL,MXSCN);

	long type=ekks_readSMPSData(stoch,core_Name,time_Name,stoch_Name);                  
	if (type<0) exit(999);
	ekks_writeNativeData(stoch,spl_Name);  

/* Describe Full model and don't generate deterministic equivalent */
	ekks_describeFullModel(stoch,0);  

/* Set decomposition solver parameters */
    ekks_setMaxsubmodels(stoch,10);                      
	ekks_setScaleOn(stoch);
	ekks_setCrash(stoch,3);
	ekks_setPresolve(stoch,3);
	ekks_setFinalSimplexSolverOff(stoch);

/* Solve using two-stage Benders decomposition */
	ekks_setCutPeriod(stoch,2);
    ekks_bendersLSolve(stoch,0);  
    
/* return optimal value */
	double objval= ekks_getRobjvalue(stoch);

/* Delete EKKStoch objects */
    ekks_deleteStoch(stoch);

/* End EKKContext*/
    ekks_endContext(env);

    return objval;
}
